# $Id: selectwin.py,v 1.8 2002/02/17 21:57:08 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import setup
import ui
import sim
import metaclient
import abswindow

class SelectWin(abswindow.AbsWindow):
    WindowNickName = "select"
    def _build(self):
        ui = abswindow.WindowList().getUserInterface()
        self._list = ui.get_widget("SimSelClist")
        self._conn = ui.get_widget("SelectWinConnect")
        self._stat = ui.get_widget("SelectWinStatMeta")        
        
        # connectors on clist
        self._list.connect('select-row', self.listRowSelect)
        self._list.connect('unselect-row', self.listRowUnSelect)
        
        # connectors on others.
        w = ui.get_widget("SelectWinConMeta")
        w.connect('clicked', self.contactMeta)
        w = ui.get_widget("SelectWinCancel")
        w.connect('clicked', self._windowCloseButton)
        self._conn.connect('clicked', self.butConnect)
        #self.contactMeta()

    def listRowSelect(self, widget, row, col, e):
        self._conn.set_sensitive(1)
        self._selRow = row
    def listRowUnSelect(self, widget, row, col, e):
        self._conn.set_sensitive(0)
    def contactMeta(self, widget=None):
        self._list.clear()
        cid  = self._stat.get_context_id("")
        self._stat.push(cid, "Connecting")
        _getMetaData(setup.url_meta_server, self._list, self._stat, cid)
    def butConnect(self, widget):
        self._window.hide()
        ior = self._list.get_row_data(self._selRow)
        sim.Simulation(ior, "tester")
    def _showServers(self, serv):
        self._list.append([serv.hostName,"",""])

class _getMetaData:
    def __init__(self, url, list, stat, cid):
        self.meta = metaclient.MetaClient()
        self.meta.connect(self.connected_cb, url)
        self.list = list
        self.stat = stat
        self.cid = cid
        self.meta.addExceptionHandler("CORBA_Failure",
                                      self.handleMetaServerMissing)
    def connected_cb(self):
        print "connected_cb called"
        self.stat.pop(self.cid)
        self.stat.push(self.cid, "Servers")
        self.meta.getServerList(self.getServerList_cb)
    def getServerList_cb(self, serverList=None):
        print "getServerList_cb called"
        if serverList:
            self.list.freeze()
            for i in serverList:
                rnr = self.list.append([i.hostName, "", ""])
                self.list.set_row_data(rnr, i.serverIor)
                self.stat.pop(self.cid)
                self.stat.push(self.cid, "Data..")
                self.meta.getServerData(self.getServerData_cb, i.serverIor)

            self.list.thaw()
        else:
            print "ServerList was empty!"
            self.stat.pop(self.cid)
            self.stat.push(self.cid, "No Servers")
    def getServerData_cb(self, data, ior):
        print "getServerData_cb called"
        self.stat.pop(self.cid)
        self.stat.push(self.cid, "Done.")
        rnr = self.list.find_row_from_data(ior)
        ver = data.simVersion
        u = "%d/%d"    % (data.currentUsers, data.maxUsers)
        v = "%s.%s.%s" % (ver.major, ver.minor, ver.patch)
        self.list.freeze()
        self.list.set_text(rnr, 1, u)
        self.list.set_text(rnr, 2, v)
        self.list.thaw()
    def handleMetaServerMissing(self, e):
        dialog = abswindow.WindowList()['dialog']
        dialog.startMetaServer()

        
# $Log: selectwin.py,v $
# Revision 1.8  2002/02/17 21:57:08  marijn
# Added generalized exception handling w/ callbacks
#
# Revision 1.7  2002/02/07 16:25:21  marijn
# small fixes
#
# Revision 1.6  2002/02/07 12:51:36  marijn
# Redid window referencing
#
# Revision 1.5  2002/02/02 17:23:57  marijn
# Updated copyright information
#
# Revision 1.4  2002/01/14 15:45:00  marijn
# Initial support for data display
#
# Revision 1.3  2001/12/26 02:33:59  marijn
# Added SimClient to threaded model
#
# Revision 1.2  2001/12/11 03:51:54  marijn
# Small bugfix
#
# Revision 1.1  2001/12/10 00:10:05  marijn
# moved to selectwin.py. Added threaded meta-server support.
#
